﻿package pers.SimpleAlarmClock;
import pers.SimpleAlarmClock.R;
import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.graphics.Typeface;
import android.media.MediaPlayer;
import android.provider.Settings;
import android.net.Uri;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.Vibrator;
import android.os.Handler;
import android.util.Log;
import android.media.AudioManager;
//Главный класс и экран (активити) приложения
public class AlarmActivity extends Activity {
 private final static String tag = "AlarmActivity"; //Тег для ведения логов в классе
 private final static int VIBRATE_DELAY_TIME = 1500; //Время между срабатываниями вибры, мс
 private final static int DURATION_OF_VIBRATION = 1000; //Продолжительность вибры, мс 
 private MediaPlayer mPlayer;
 private PowerManager.WakeLock mWakeLock;
 private boolean mEnableVibration;
 private Handler mPerformVibrationHandler = new Handler();
 private Runnable mVibrationRunnable = new Runnable() {
  @Override
  public void run() {
   Vibrator v = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);   //Получить вхождение вибратора из текущего контекста
   v.vibrate(DURATION_OF_VIBRATION);
   mPerformVibrationHandler.postDelayed(mVibrationRunnable,DURATION_OF_VIBRATION+VIBRATE_DELAY_TIME); //Установить цикл для вибры
  }
 };

 @Override
 public void onCreate(Bundle savedInstanceState) {
  //Вызывается, когда активити создаётся впервые	
  super.onCreate(savedInstanceState);
  Log.d(tag, "onCreate"); //Будем логгировать что можно
  //Будим телефон, чтоб показать нашу активити
  PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
  mWakeLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK | PowerManager.ACQUIRE_CAUSES_WAKEUP | PowerManager.ON_AFTER_RELEASE, tag);
  mWakeLock.acquire();
  requestWindowFeature(Window.FEATURE_NO_TITLE);
  getWindow().addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED);
  setContentView(R.layout.act_popout);
  Typeface t = Typeface.createFromAsset(getAssets(),"fonts/font.ttf");
  TextView tv = (TextView) findViewById(R.id.tv_stop_player);
  tv.setTypeface(t);
  //Получаем установки, чтоб поставить сигнал будильника
  SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
  String ringtonePref = prefs.getString("ringtone_pref",Settings.System.DEFAULT_RINGTONE_URI.toString());
  if (prefs.getBoolean("vibration_pref", false)) {
   Log.d(tag, "Vibration is set");
   mEnableVibration = true; 
  }
  mPlayer = new MediaPlayer();    
  mPlayer.setOnErrorListener(new MediaPlayer.OnErrorListener() {
   @Override
   public boolean onError(MediaPlayer mp, int what, int extra) {
    //Обработчик ошибок медиаплеера
    Log.d(tag, "Error occurred while playing audio");
    Log.d(tag, "Error code: " + what);
    mp.stop();
    mp.release();
    return true;
   }    
  });

  try { //Установка плеера
   mPlayer.setDataSource(this, Uri.parse(ringtonePref));
  } 
  catch (Exception e) {
   Log.d(tag, "Setup player exception: " + e.toString());
  }
 }

 @Override
 protected void onResume() {
  //Вызывается перед тем, как перед тем как активити будет доступно для действий пользователя 
  super.onResume();
  Log.d(tag, "onResume");
  try { //Этот цикл будет вечным :)
   mPlayer.setLooping(true);
   mPlayer.setAudioStreamType(AudioManager.STREAM_ALARM);
   mPlayer.prepare();
   mPlayer.start();
   if (mEnableVibration) { //Это начинается вибра
    mPerformVibrationHandler.post(mVibrationRunnable);
   }
  } 
  catch (Exception e) {
   Log.d(tag, "Start player is failed:" + e.toString());
   if (mPlayer.isPlaying()) mPlayer.stop();
  }
 }

 @Override
 protected void onStart() {
  //Вызывается перед тем, как активити будет видно пользователю
  super.onStart();
  Log.d(tag, "onStart");
 }

 @Override
 protected void onPause() {
  //вызывается перед тем, как будет показано другое активити
  super.onPause();
  Log.d(tag, "onPause");
  dismissAlarm(); //заткнуть звонилку
 }
        
 @Override
 protected void onDestroy() {
  //Вызывается перед удалением активити
  super.onDestroy();
  Log.d(tag, "onDestroy");
  mWakeLock.release();
 }
        
 public void stopPlayer(View v) {
  //Вызывается при тапе на экран для затыкания будильника
  //Оформление экрана - в layout
  dismissAlarm();
  finish();
 }

 private void dismissAlarm() { //Затыкание звонилки
  if (mPlayer.isPlaying()) mPlayer.stop();
  mPerformVibrationHandler.removeCallbacks(mVibrationRunnable);   
 }
}