package pers.SimpleAlarmClock;
import pers.SimpleAlarmClock.R;
import android.app.Activity;
import android.os.Bundle;
import android.content.Intent;
import android.app.PendingIntent;
import android.app.AlarmManager;
import android.view.View;
import android.view.Window;
import android.widget.Toast;
import android.widget.TextView;
import android.widget.FrameLayout;
import android.view.MenuItem;
import android.view.MenuInflater;
import android.view.Menu;
import android.graphics.Typeface;
import android.os.Handler;
import java.util.Calendar;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.net.Uri;
import android.util.Log;

public class SimpleAlarmClock extends Activity
 implements View.OnClickListener, ClockView.OnAlarmSetListener {
 private final static String tag = "SimpleAlarmClock";
 private final static long NOTIFICATION_DELAY_TIME = 2000;
 private final static int PREFS_UPDATED = 100;
 public final static String PREFS_NAME = "SimpleAlarmClockSharedPreferences";
 public final static int BLACK = 0xff000000;
 public final static int WHITE = 0xffffffff;
 private ClockView mHoursClock; 
 private ClockView mMinutesClock; //Часы и минуты
 private SwitchButton mSwitchBtn; //День/Ночь 
 private TextView mStartTv;
 private FrameLayout mContainerLayout; 
       
 private Handler mShowToastHandler = new Handler(); 
 private Runnable mToastRunnable = new Runnable() {
  //Для управления окошечком с сообщением о времени до срабатывания
  @Override
  public void run() { //Ничего не делать
  }
 };
    
 @Override
 public void onCreate(Bundle savedInstanceState) { //Создание
  super.onCreate(savedInstanceState);
  requestWindowFeature(Window.FEATURE_NO_TITLE);
  setContentView(R.layout.act_main);
  Typeface t = Typeface.createFromAsset(getAssets(),"fonts/font.ttf");
  mContainerLayout = (FrameLayout) findViewById(R.id.l_container);
  mStartTv = (TextView) findViewById(R.id.tv_invite);
  mStartTv.setTypeface(t);
  mHoursClock = (ClockView) findViewById(R.id.v_hours);
  mMinutesClock = (ClockView) findViewById(R.id.v_minutes);
  mSwitchBtn = (SwitchButton) findViewById(R.id.btn_switch_meridiem);
  //Для активити с информацией об установке будильника
  mHoursClock.setOnAlarmSetListener(this);
  mMinutesClock.setOnAlarmSetListener(this);
  mSwitchBtn.setOnClickListener(this);
  //Восстановить последнее время срабатывания из SharedPreferences
  SharedPreferences prefs = getSharedPreferences(PREFS_NAME, 0);
  mHoursClock.setTime(prefs.getInt("hour", 8));
  mMinutesClock.setTime(prefs.getInt("minute", 0));
  mSwitchBtn.setChecked(prefs.getBoolean("daytime", true));
  //Если не установлен - установить
  if (prefs.getBoolean("is_alarm_on", false)) {
   mContainerLayout.setPadding(0, 0, 0, 0);
   mStartTv.setVisibility(View.GONE);
  }
  updateColorScheme();
  // 
  SharedPreferences defPref = PreferenceManager.getDefaultSharedPreferences(this);
  String ringtone = defPref.getString("ringtone_pref", 
  Settings.System.DEFAULT_RINGTONE_URI.toString());
  Uri uri = Uri.parse(ringtone);
  if (uri.equals(Settings.System.DEFAULT_RINGTONE_URI)) {
   defPref.edit().putString("ringtone_pref",Settings.System.DEFAULT_RINGTONE_URI.toString()).commit();
  }
  mMinutesClock.setSnap(defPref.getBoolean("snap_pref", true));
 }

 @Override
 public boolean onCreateOptionsMenu(Menu menu) {
  MenuInflater inflater = getMenuInflater();
  inflater.inflate(R.menu.options_menu, menu);
  return true;
 }

 @Override
 public boolean onOptionsItemSelected(MenuItem item) {
  switch (item.getItemId()) {
   case R.id.om_stop: //Выключить будильник, показать уведомление
    mContainerLayout.setPadding(0, 0, 0, 0);
    mStartTv.setVisibility(View.VISIBLE);
    stopRepeating();
   break;
   case R.id.om_prefs: //Запустить форму Preferences
    Intent intent = new Intent(getBaseContext(),Preferences.class);
    startActivityForResult(intent, PREFS_UPDATED);
   break;
  }
  return super.onOptionsItemSelected(item);
 }

 @Override
 protected void onActivityResult(int requestCode, int resultCode, Intent data) {
  switch (requestCode) {
   case PREFS_UPDATED:
    SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
    mMinutesClock.setSnap(prefs.getBoolean("snap_pref", true));
   break;
  }
 }

 @Override 
 protected void onPause() {
  super.onPause();
  SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);
  SharedPreferences.Editor editor = settings.edit();
  //Сохранить время будильника 
  editor.putInt("hour", mHoursClock.getTime());
  editor.putInt("minute", mMinutesClock.getTime());
  editor.putBoolean("daytime", mSwitchBtn.isChecked());
  editor.putBoolean("is_alarm_on", mStartTv.getVisibility() == View.GONE);
  editor.commit();
 }

 public void showAlarm(View v) {
  Log.d(tag, "TEXTVIEW ONCLICK");
  v.setVisibility(View.GONE);
  mContainerLayout.setPadding(10, 10, 10, 10);
  //По умолчанию ставим на текущее время
  setAlarm(mHoursClock.getTime(), mMinutesClock.getTime());
 }
      
 @Override
 public void onClick(View v) { //Переключалка день/ночь
  ((SwitchButton) v).switchState();
  updateColorScheme();
  //setAlarm(mHoursClock.getTime(), mMinutesClock.getTime());
 }

 private void updateColorScheme() {
  FrameLayout layout = (FrameLayout) findViewById(R.id.l_container);      
  layout.setBackgroundColor(mSwitchBtn.isChecked() ? WHITE : BLACK);
  mHoursClock.updateColors(mSwitchBtn.isChecked() ? BLACK : WHITE);
  mMinutesClock.updateColors(mSwitchBtn.isChecked() ? BLACK : WHITE);
 }

 @Override
 public void onSetTime (ClockView v, int time) {
  int hour = 0;
  int minute = 0;
  switch (v.getId()) {
   case R.id.v_hours:
    hour = time;
    minute = mMinutesClock.getTime();
   break;
   case R.id.v_minutes:
    hour = mHoursClock.getTime();
    minute = time;
   break;
  }
  final int h = hour;
  final int m = minute;
  mShowToastHandler.removeCallbacks(mToastRunnable);
  mToastRunnable = new Runnable() {
   @Override
   public void run() { setAlarm(h, m); }
  };
  mShowToastHandler.postDelayed(mToastRunnable, NOTIFICATION_DELAY_TIME);
 }

 @Override
 public void onProceedSetTime() {
  mShowToastHandler.removeCallbacks(mToastRunnable);
 }

 private void setAlarm(int hour, int minute) {
  //Вычисляет пеиод времени для будильника и запускает его
  //hour = (mSwitchBtn.isChecked() ? (hour % 12) : (hour % 12) + 12);
  long now = System.currentTimeMillis();
  Calendar current = Calendar.getInstance();
  current.setTimeInMillis(now);
  Calendar alarm = (Calendar) current.clone();  
  alarm.set(Calendar.HOUR_OF_DAY, hour);
  alarm.set(Calendar.MINUTE, minute);
  alarm.set(Calendar.SECOND, 0);
  if (current.getTimeInMillis() > alarm.getTimeInMillis()) {
   //Время звонка в мс меньше текущего в мс - добавить в календарь сутки    
   alarm.add(Calendar.DAY_OF_MONTH, 1);
  }
  //Вычисляем время до будильника 
  long delay = alarm.getTimeInMillis() - current.getTimeInMillis();
  final int hours = (int) ((delay / (1000 * 60 * 60)) % 24);
  final int minutes = (int) ((delay / (1000 * 60)) % 60);
  Log.d(tag, "Alarm is set on "+alarm.get(Calendar.DAY_OF_MONTH)+" at "+ alarm.get(Calendar.HOUR)+":"+alarm.get(Calendar.MINUTE));
  stopRepeating(); //Останавливаем имеющийся будильник
  startRepeating(alarm); //Запускаем новый
  //Уведомить пользователя о срабатывании будильника
  mShowToastHandler.removeCallbacks(mToastRunnable);
  mToastRunnable = new Runnable() {
   @Override
   public void run() {
	String zero = "0";
	Toast.makeText(SimpleAlarmClock.this, "Будильник сработает через "+hours+" ч. "+(minutes<10 ? zero.concat(""+minutes) : minutes)+" мин.", Toast.LENGTH_LONG).show();
   }
  };
  mShowToastHandler.postDelayed(mToastRunnable, NOTIFICATION_DELAY_TIME);
 }

 private void startRepeating(Calendar alarmTime) {
  //Пора включать - послать PendingIntent для BroadcastReceiver
  Log.d(tag, "Start repeating");
  //Получатель прописан в AndroidManifest.xml
  Intent intent = new Intent(this, AlarmReceiver.class);
  PendingIntent sender = PendingIntent.getBroadcast(this, 0, intent, 0);
  AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
  am.setRepeating(AlarmManager.RTC_WAKEUP, alarmTime.getTimeInMillis(), (24 * 60 * 60 * 1000), sender);
 }

 private void stopRepeating() {
  Log.d(tag, "Stop repeating");
  Intent intent = new Intent (this, AlarmReceiver.class);
  PendingIntent sender = PendingIntent.getBroadcast(this, 0, intent, 0);
  AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
  am.cancel(sender); //Выключить будильник
 }
}